@@ -72,21 +72,21 @@ static void hex_string_to_bytes(uint8_t* hexstr, int length, uint8_t* bytes)
 // of PyUnicode_for string data and PyBytes_ for binary data:
 PyObject* PyUnicodeString_FromString(const char *str) {
   PyObject* res;
-  #if PY_MAJOR_VERSION < 3
-    res = PyString_FromString(str);
-  #else
-    res = PyUnicode_FromString(str);
-  #endif
+#if PY_MAJOR_VERSION < 3
+  res = PyString_FromString(str);
+#else
+  res = PyUnicode_FromString(str);
+#endif
   return res;
 }
 
 PyObject* PyByteString_FromString(const char *str) {
   PyObject* res;
-  #if PY_MAJOR_VERSION < 3
-    res = PyString_FromString(str);
-  #else
-    res = PyBytes_FromString(str);
-  #endif
+#if PY_MAJOR_VERSION < 3
+  res = PyString_FromString(str);
+#else
+  res = PyBytes_FromString(str);
+#endif
   return res;  
 }
 
@@ -100,6 +100,8 @@ static void callback_friend_request(uint8_t* public_key, uint8_t* data,
     uint16_t length, void* self)
 {
   uint8_t buf[TOX_FRIEND_ADDRESS_SIZE * 2 + 1];
+  memset(buf, 0, TOX_FRIEND_ADDRESS_SIZE * 2 + 1);
+
   bytes_to_hex_string(public_key, TOX_FRIEND_ADDRESS_SIZE, buf);
 
   PyObject_CallMethod((PyObject*)self, "on_friend_request", "ss#", buf, data,
@@ -158,8 +160,12 @@ static void callback_connection_status(Tox *tox, int friendnumber,
 static void callback_group_invite(Tox *tox, int friendnumber,
     uint8_t* group_public_key, void *self)
 {
-  PyObject_CallMethod((PyObject*)self, "on_group_invite", "is#", friendnumber,
-      group_public_key, TOX_FRIEND_ADDRESS_SIZE);
+  uint8_t gpk[TOX_CLIENT_ID_SIZE * 2 + 1];
+  memset(gpk, 0, TOX_CLIENT_ID_SIZE * 2 + 1);
+
+  bytes_to_hex_string(group_public_key, TOX_CLIENT_ID_SIZE, gpk);
+  PyObject_CallMethod((PyObject*)self, "on_group_invite", "is", friendnumber,
+      gpk);
 }
 
 static void callback_group_message(Tox *tox, int groupid,
@@ -188,15 +194,25 @@ static void callback_file_control(Tox *m, int friendnumber,
     uint8_t receive_send, uint8_t filenumber, uint8_t control_type,
     uint8_t* data, uint16_t length, void* self)
 {
+#if PY_MAJOR_VERSION < 3
   PyObject_CallMethod((PyObject*)self, "on_file_control", "iiiis#",
       friendnumber, receive_send, filenumber, control_type, data, length);
+#else
+  PyObject_CallMethod((PyObject*)self, "on_file_control", "iiiiy#",
+      friendnumber, receive_send, filenumber, control_type, data, length);
+#endif
 }
 
 static void callback_file_data(Tox *m, int friendnumber, uint8_t filenumber,
     uint8_t* data, uint16_t length, void* self)
 {
+#if PY_MAJOR_VERSION < 3
   PyObject_CallMethod((PyObject*)self, "on_file_data", "iis#",
       friendnumber, filenumber, data, length);
+#else
+  PyObject_CallMethod((PyObject*)self, "on_file_data", "iiy#",
+      friendnumber, filenumber, data, length);
+#endif
 }
 
 static int init_helper(ToxCore* self, PyObject* args)
@@ -277,6 +293,7 @@ ToxCore_getaddress(ToxCore* self, PyObject* args)
 {
   uint8_t address[TOX_FRIEND_ADDRESS_SIZE];
   uint8_t address_hex[TOX_FRIEND_ADDRESS_SIZE * 2 + 1];
+  memset(address_hex, 0, TOX_FRIEND_ADDRESS_SIZE * 2 + 1);
 
   tox_get_address(self->tox, address);
   bytes_to_hex_string(address, TOX_FRIEND_ADDRESS_SIZE, address_hex);
@@ -355,12 +372,14 @@ ToxCore_add_friend_norequest(ToxCore* self, PyObject* args)
   uint8_t pk[TOX_FRIEND_ADDRESS_SIZE];
   hex_string_to_bytes(address, TOX_FRIEND_ADDRESS_SIZE, pk);
 
-  if (tox_add_friend_norequest(self->tox, pk) == -1) {
+  // return friend_id on success, else -1.
+  int res = tox_add_friend_norequest(self->tox, pk);
+  if (res == -1) {
     PyErr_SetString(ToxCoreError, "failed to add friend");
     return NULL;
   }
 
-  Py_RETURN_NONE;
+  return PyLong_FromLong(res);
 }
 
 static PyObject*
@@ -389,9 +408,10 @@ static PyObject*
 ToxCore_get_client_id(ToxCore* self, PyObject* args)
 {
   uint8_t pk[TOX_CLIENT_ID_SIZE + 1];
-  pk[TOX_FRIEND_ADDRESS_SIZE] = 0;
+  pk[TOX_CLIENT_ID_SIZE] = 0;
 
   uint8_t hex[TOX_CLIENT_ID_SIZE * 2 + 1];
+  memset(hex, 0, TOX_CLIENT_ID_SIZE * 2 + 1);
 
   int friendid = 0;
 
@@ -607,7 +627,7 @@ ToxCore_set_status_message(ToxCore* self, PyObject* args)
 }
 
 static PyObject*
-ToxCore_set_userstatus(ToxCore* self, PyObject* args)
+ToxCore_set_user_status(ToxCore* self, PyObject* args)
 {
   int status = 0;
 
@@ -615,7 +635,7 @@ ToxCore_set_userstatus(ToxCore* self, PyObject* args)
     return NULL;
   }
 
-  if (tox_set_userstatus(self->tox, status) == -1) {
+  if (tox_set_user_status(self->tox, status) == -1) {
     PyErr_SetString(ToxCoreError, "failed to set status");
     return NULL;
   }
@@ -810,14 +830,17 @@ ToxCore_invite_friend(ToxCore* self, PyObject* args)
 static PyObject*
 ToxCore_join_groupchat(ToxCore* self, PyObject* args)
 {
-  uint8_t* pk = NULL;
+  uint8_t* public_key = NULL;
   int length = 0;
   int friendnumber = 0;
+  uint8_t pk[TOX_CLIENT_ID_SIZE];
 
-  if (!PyArg_ParseTuple(args, "is#", &friendnumber, &pk, &length)) {
+  if (!PyArg_ParseTuple(args, "is#", &friendnumber, &public_key, &length)) {
     return NULL;
   }
 
+  hex_string_to_bytes(public_key, TOX_CLIENT_ID_SIZE, pk);
+
   int ret = tox_join_groupchat(self->tox, friendnumber, pk);
   if (ret == -1) {
     PyErr_SetString(PyExc_TypeError, "failed to join group chat");
@@ -1042,19 +1065,21 @@ static PyObject*
 ToxCore_bootstrap_from_address(ToxCore* self, PyObject* args)
 {
   int ipv6enabled = TOX_ENABLE_IPV6_DEFAULT;
-  int port = 0;
+  uint16_t port = 0;
   uint8_t* public_key = NULL;
   char* address = NULL;
   int addr_length = 0;
   int pk_length = 0;
+  uint8_t pk[TOX_CLIENT_ID_SIZE];
 
-  if (!PyArg_ParseTuple(args, "s#iis#", &address, &addr_length, &ipv6enabled,
+  if (!PyArg_ParseTuple(args, "s#iHs#", &address, &addr_length, &ipv6enabled,
         &port, &public_key, &pk_length)) {
     return NULL;
   }
 
-  int ret = tox_bootstrap_from_address(self->tox, address, ipv6enabled, port,
-      public_key);
+  hex_string_to_bytes(public_key, TOX_CLIENT_ID_SIZE, pk);
+  int ret = tox_bootstrap_from_address(self->tox, address, ipv6enabled,
+      htons(port), pk);
 
   if (!ret) {
     PyErr_SetString(ToxCoreError, "failed to resolve address");
@@ -1087,6 +1112,9 @@ ToxCore_do(ToxCore* self, PyObject* args)
 {
   tox_do(self->tox);
 
+  if (PyErr_Occurred()) {
+    return NULL;
+  }
   Py_RETURN_NONE;
 }
 
@@ -1113,11 +1141,11 @@ ToxCore_save(ToxCore* self, PyObject* args)
 
   // The PyString_* functions were split in Python 3.* into 
   // PyUnicode_ for string data and PyBytes_ for binary data
-  #if PY_MAJOR_VERSION < 3
-    res = PyString_FromStringAndSize((const char*)buf, size);
-  #else
-    res = PyBytes_FromStringAndSize((const char*)buf, size);
-  #endif
+#if PY_MAJOR_VERSION < 3
+  res = PyString_FromStringAndSize((const char*)buf, size);
+#else
+  res = PyBytes_FromStringAndSize((const char*)buf, size);
+#endif
   free(buf);
 
   return res;
@@ -1246,7 +1274,7 @@ PyMethodDef Tox_methods[] = {
     "Callback for receiving friend status changes, default implementation "
     "does nothing.\n\n"
     ".. seealso ::\n"
-    "    :meth:`.set_userstatus`"
+    "    :meth:`.set_user_status`"
   },
   {
     "on_read_receipt", (PyCFunction)ToxCore_callback_stub, METH_VARARGS,
@@ -1396,8 +1424,8 @@ PyMethodDef Tox_methods[] = {
     "Set our user status message."
   },
   {
-    "set_userstatus", (PyCFunction)ToxCore_set_userstatus, METH_VARARGS,
-    "set_userstatus(status)\n"
+    "set_user_status", (PyCFunction)ToxCore_set_user_status, METH_VARARGS,
+    "set_user_status(status)\n"
     "Set our user status, status can have following values:\n\n"
     "+------------------------+--------------------+\n"
     "| kind                   | description        |\n"
@@ -1435,7 +1463,7 @@ PyMethodDef Tox_methods[] = {
     "get_user_status(friend_number)\n"
     "Get friend status.\n\n"
     ".. seealso ::\n"
-    "    :meth:`.set_userstatus`"
+    "    :meth:`.set_user_status`"
   },
   {
     "get_self_user_status", (PyCFunction)ToxCore_get_self_user_status,
@@ -1443,7 +1471,7 @@ PyMethodDef Tox_methods[] = {
     "get_self_user_status()\n"
     "Get user status of youself.\n\n"
     ".. seealso ::\n"
-    "    :meth:`.set_userstatus`"
+    "    :meth:`.set_user_status`"
   },
   {
     "set_sends_receipts", (PyCFunction)ToxCore_set_send_receipts,
