@@ -26,7 +26,6 @@
 
 extern PyTypeObject ToxCoreType;
 extern PyObject* ToxCoreError;
-extern PyMethodDef Tox_methods[];
 
 extern void ToxCore_install_dict();
 
@@ -36,7 +35,7 @@ struct PyModuleDef moduledef = {
   "tox",
   "Python Toxcore module",
   -1,
-  Tox_methods,
+  NULL,
   NULL,
   NULL,
   NULL,
@@ -45,10 +44,12 @@ struct PyModuleDef moduledef = {
 #endif
 
 #if PY_MAJOR_VERSION >= 3
-PyObject *PyInit_tox(void) {
+PyMODINIT_FUNC PyInit_tox(void)
+{
   PyObject *m = PyModule_Create(&moduledef);
 #else
-PyMODINIT_FUNC inittox(void) {
+PyMODINIT_FUNC inittox(void)
+{
   PyObject *m = Py_InitModule("tox", NULL);
 #endif
 
